@@ -8,10 +8,12 @@ Replays an existing analysis onto the sheet
 """
 
 from typing import Any, Dict
+from mitosheet.errors import make_no_analysis_error
+
+from mitosheet.mito_analytics import log
 from mitosheet.saved_analyses import read_and_upgrade_analysis
 from mitosheet.types import StepsManagerType
 
-
 REPLAY_ANALYSIS_UPDATE_EVENT = 'replay_analysis_update'
 REPLAY_ANALYSIS_UPDATE_PARAMS = [
     'analysis_name',
@@ -36,6 +38,16 @@ def execute_replay_analysis_update(
     state container (except the initalize step) before applying the saved analysis.
     """
 
+    # If we're getting an event telling us to update, we read in the steps from the file
+    analysis = read_and_upgrade_analysis(analysis_name)
+
+    # If there is no analysis with this name, generate an error. This must occur before  
+    # steps_manager.steps = steps_manager.steps[:1] so that we don't clear all of the user's generated code. 
+    # This is particularly important for users who received the notebook from a colleague.
+    if analysis is None:
+        log('replayed_nonexistant_analysis_failed')
+        raise make_no_analysis_error(analysis_name)
+
     # We only keep the intialize step only, if we want to clear,
     # and also update the analysis name to the replayed analysis
     # NOTE: we update the analysis name so that when the code
@@ -46,12 +58,6 @@ def execute_replay_analysis_update(
         steps_manager.steps = steps_manager.steps[:1]
         steps_manager.analysis_name = analysis_name
 
-    # If we're getting an event telling us to update, we read in the steps from the file
-    analysis = read_and_upgrade_analysis(analysis_name)
-    # If there is no analysis with this name, give up
-    if analysis is None:
-        return
-
     # When replaying an analysis with import events, you can also send over
     # new params to the import events to replace them. We replace them in the steps here
     if import_summaries is not None:
@@ -83,4 +89,4 @@ REPLAY_ANALYSIS_UPDATE = {
     'event_type': REPLAY_ANALYSIS_UPDATE_EVENT,
     'params': REPLAY_ANALYSIS_UPDATE_PARAMS,
     'execute': execute_replay_analysis_update
-}
\ No newline at end of file
+}
